@@ -3,7 +3,7 @@ BEGIN {
   $App::DuckPAN::AUTHORITY = 'cpan:DDG';
 }
 # ABSTRACT: The DuckDuckGo DuckPAN client
-$App::DuckPAN::VERSION = '0.137';
+$App::DuckPAN::VERSION = '0.138';
 
 use Moo;
 use MooX::Cmd;
@@ -24,6 +24,7 @@ use Term::ReadLine;
 use Carp;
 use Encode;
 use Path::Class;
+use File::Path qw(remove_tree);
 
 our $VERSION ||= '9.999';
 
@@ -177,6 +178,13 @@ sub execute {
 			} elsif ($_ =~ m/^(duckpan|upgrade|update)$/i) {
 				push @modules, 'App::DuckPAN';
 				push @modules, 'DDG' if lc($_) eq 'upgrade';
+				my $cache = $self->cfg->cache_path;
+				if (-d $cache){
+					print "Clearing DuckPAN cache...";
+					print `rm -r $cache/*`;
+					# remove_tree($self->cfg->cache_path); #better option, not working though...
+					print "Done\n";
+				}
 			} else {
 				push @left_args, $_;
 			}
@@ -372,7 +380,7 @@ App::DuckPAN - The DuckDuckGo DuckPAN client
 
 =head1 VERSION
 
-version 0.137
+version 0.138
 
 =head1 DuckPAN
 
